from typing import List

from openapi_client.models import OrmKernelImage, ResponseKernelImage
from savvihub import vessl_api
from savvihub.organization import _get_organization_name


def read_kernel_image(image_id: int) -> ResponseKernelImage:
    return vessl_api.kernel_image_read_api(image_id=image_id)


def list_kernel_images(**kwargs) -> List[OrmKernelImage]:
    """List kernel images

    Keyword args:
        organization_name (str): override default organization
    """
    return vessl_api.kernel_image_list_api(
        organization_name=_get_organization_name(**kwargs),
    ).results



# coding: utf-8

"""
    Aron API

    No description provided (generated by Openapi Generator https://github.com/openapitools/openapi-generator)  # noqa: E501

    The version of the OpenAPI document: 1.0.0
    Generated by: https://openapi-generator.tech
"""


try:
    from inspect import getfullargspec
except ImportError:
    from inspect import getargspec as getfullargspec
import pprint
import re  # noqa: F401
import six

from openapi_client.configuration import Configuration


class ResponseWorkspaceList(object):
    """NOTE: This class is auto generated by OpenAPI Generator.
    Ref: https://openapi-generator.tech

    Do not edit the class manually.
    """

    """
    Attributes:
      openapi_types (dict): The key is attribute name
                            and the value is attribute type.
      attribute_map (dict): The key is attribute name
                            and the value is json key in definition.
    """
    openapi_types = {
        'backup_volume_id': 'int',
        'created_by': 'ResponseSimpleUser',
        'created_dt': 'datetime',
        'endpoints': 'ResponseWorkloadEndpoints',
        'histories': 'list[ResponseWorkloadHistoryInfo]',
        'id': 'int',
        'kernel_cluster': 'ResponseKernelCluster',
        'kernel_resource_spec': 'ResponseKernelResourceSpec',
        'last_backup_dt': 'datetime',
        'name': 'str',
        'status': 'str',
        'status_last_updated': 'datetime',
        'status_reason': 'str',
        'updated_dt': 'datetime'
    }

    attribute_map = {
        'backup_volume_id': 'backup_volume_id',
        'created_by': 'created_by',
        'created_dt': 'created_dt',
        'endpoints': 'endpoints',
        'histories': 'histories',
        'id': 'id',
        'kernel_cluster': 'kernel_cluster',
        'kernel_resource_spec': 'kernel_resource_spec',
        'last_backup_dt': 'last_backup_dt',
        'name': 'name',
        'status': 'status',
        'status_last_updated': 'status_last_updated',
        'status_reason': 'status_reason',
        'updated_dt': 'updated_dt'
    }

    def __init__(self, backup_volume_id=None, created_by=None, created_dt=None, endpoints=None, histories=None, id=None, kernel_cluster=None, kernel_resource_spec=None, last_backup_dt=None, name=None, status=None, status_last_updated=None, status_reason=None, updated_dt=None, local_vars_configuration=None):  # noqa: E501
        """ResponseWorkspaceList - a model defined in OpenAPI"""  # noqa: E501
        if local_vars_configuration is None:
            local_vars_configuration = Configuration.get_default_copy()
        self.local_vars_configuration = local_vars_configuration

        self._backup_volume_id = None
        self._created_by = None
        self._created_dt = None
        self._endpoints = None
        self._histories = None
        self._id = None
        self._kernel_cluster = None
        self._kernel_resource_spec = None
        self._last_backup_dt = None
        self._name = None
        self._status = None
        self._status_last_updated = None
        self._status_reason = None
        self._updated_dt = None
        self.discriminator = None

        self.backup_volume_id = backup_volume_id
        self.created_by = created_by
        self.created_dt = created_dt
        self.endpoints = endpoints
        self.histories = histories
        self.id = id
        if kernel_cluster is not None:
            self.kernel_cluster = kernel_cluster
        self.kernel_resource_spec = kernel_resource_spec
        self.last_backup_dt = last_backup_dt
        self.name = name
        self.status = status
        self.status_last_updated = status_last_updated
        self.status_reason = status_reason
        self.updated_dt = updated_dt

    @property
    def backup_volume_id(self):
        """Gets the backup_volume_id of this ResponseWorkspaceList.  # noqa: E501


        :return: The backup_volume_id of this ResponseWorkspaceList.  # noqa: E501
        :rtype: int
        """
        return self._backup_volume_id

    @backup_volume_id.setter
    def backup_volume_id(self, backup_volume_id):
        """Sets the backup_volume_id of this ResponseWorkspaceList.


        :param backup_volume_id: The backup_volume_id of this ResponseWorkspaceList.  # noqa: E501
        :type backup_volume_id: int
        """
        if self.local_vars_configuration.client_side_validation and backup_volume_id is None:  # noqa: E501
            raise ValueError("Invalid value for `backup_volume_id`, must not be `None`")  # noqa: E501

        self._backup_volume_id = backup_volume_id

    @property
    def created_by(self):
        """Gets the created_by of this ResponseWorkspaceList.  # noqa: E501


        :return: The created_by of this ResponseWorkspaceList.  # noqa: E501
        :rtype: ResponseSimpleUser
        """
        return self._created_by

    @created_by.setter
    def created_by(self, created_by):
        """Sets the created_by of this ResponseWorkspaceList.


        :param created_by: The created_by of this ResponseWorkspaceList.  # noqa: E501
        :type created_by: ResponseSimpleUser
        """
        if self.local_vars_configuration.client_side_validation and created_by is None:  # noqa: E501
            raise ValueError("Invalid value for `created_by`, must not be `None`")  # noqa: E501

        self._created_by = created_by

    @property
    def created_dt(self):
        """Gets the created_dt of this ResponseWorkspaceList.  # noqa: E501


        :return: The created_dt of this ResponseWorkspaceList.  # noqa: E501
        :rtype: datetime
        """
        return self._created_dt

    @created_dt.setter
    def created_dt(self, created_dt):
        """Sets the created_dt of this ResponseWorkspaceList.


        :param created_dt: The created_dt of this ResponseWorkspaceList.  # noqa: E501
        :type created_dt: datetime
        """
        if self.local_vars_configuration.client_side_validation and created_dt is None:  # noqa: E501
            raise ValueError("Invalid value for `created_dt`, must not be `None`")  # noqa: E501

        self._created_dt = created_dt

    @property
    def endpoints(self):
        """Gets the endpoints of this ResponseWorkspaceList.  # noqa: E501


        :return: The endpoints of this ResponseWorkspaceList.  # noqa: E501
        :rtype: ResponseWorkloadEndpoints
        """
        return self._endpoints

    @endpoints.setter
    def endpoints(self, endpoints):
        """Sets the endpoints of this ResponseWorkspaceList.


        :param endpoints: The endpoints of this ResponseWorkspaceList.  # noqa: E501
        :type endpoints: ResponseWorkloadEndpoints
        """
        if self.local_vars_configuration.client_side_validation and endpoints is None:  # noqa: E501
            raise ValueError("Invalid value for `endpoints`, must not be `None`")  # noqa: E501

        self._endpoints = endpoints

    @property
    def histories(self):
        """Gets the histories of this ResponseWorkspaceList.  # noqa: E501


        :return: The histories of this ResponseWorkspaceList.  # noqa: E501
        :rtype: list[ResponseWorkloadHistoryInfo]
        """
        return self._histories

    @histories.setter
    def histories(self, histories):
        """Sets the histories of this ResponseWorkspaceList.


        :param histories: The histories of this ResponseWorkspaceList.  # noqa: E501
        :type histories: list[ResponseWorkloadHistoryInfo]
        """
        if self.local_vars_configuration.client_side_validation and histories is None:  # noqa: E501
            raise ValueError("Invalid value for `histories`, must not be `None`")  # noqa: E501

        self._histories = histories

    @property
    def id(self):
        """Gets the id of this ResponseWorkspaceList.  # noqa: E501


        :return: The id of this ResponseWorkspaceList.  # noqa: E501
        :rtype: int
        """
        return self._id

    @id.setter
    def id(self, id):
        """Sets the id of this ResponseWorkspaceList.


        :param id: The id of this ResponseWorkspaceList.  # noqa: E501
        :type id: int
        """
        if self.local_vars_configuration.client_side_validation and id is None:  # noqa: E501
            raise ValueError("Invalid value for `id`, must not be `None`")  # noqa: E501

        self._id = id

    @property
    def kernel_cluster(self):
        """Gets the kernel_cluster of this ResponseWorkspaceList.  # noqa: E501


        :return: The kernel_cluster of this ResponseWorkspaceList.  # noqa: E501
        :rtype: ResponseKernelCluster
        """
        return self._kernel_cluster

    @kernel_cluster.setter
    def kernel_cluster(self, kernel_cluster):
        """Sets the kernel_cluster of this ResponseWorkspaceList.


        :param kernel_cluster: The kernel_cluster of this ResponseWorkspaceList.  # noqa: E501
        :type kernel_cluster: ResponseKernelCluster
        """

        self._kernel_cluster = kernel_cluster

    @property
    def kernel_resource_spec(self):
        """Gets the kernel_resource_spec of this ResponseWorkspaceList.  # noqa: E501


        :return: The kernel_resource_spec of this ResponseWorkspaceList.  # noqa: E501
        :rtype: ResponseKernelResourceSpec
        """
        return self._kernel_resource_spec

    @kernel_resource_spec.setter
    def kernel_resource_spec(self, kernel_resource_spec):
        """Sets the kernel_resource_spec of this ResponseWorkspaceList.


        :param kernel_resource_spec: The kernel_resource_spec of this ResponseWorkspaceList.  # noqa: E501
        :type kernel_resource_spec: ResponseKernelResourceSpec
        """
        if self.local_vars_configuration.client_side_validation and kernel_resource_spec is None:  # noqa: E501
            raise ValueError("Invalid value for `kernel_resource_spec`, must not be `None`")  # noqa: E501

        self._kernel_resource_spec = kernel_resource_spec

    @property
    def last_backup_dt(self):
        """Gets the last_backup_dt of this ResponseWorkspaceList.  # noqa: E501


        :return: The last_backup_dt of this ResponseWorkspaceList.  # noqa: E501
        :rtype: datetime
        """
        return self._last_backup_dt

    @last_backup_dt.setter
    def last_backup_dt(self, last_backup_dt):
        """Sets the last_backup_dt of this ResponseWorkspaceList.


        :param last_backup_dt: The last_backup_dt of this ResponseWorkspaceList.  # noqa: E501
        :type last_backup_dt: datetime
        """

        self._last_backup_dt = last_backup_dt

    @property
    def name(self):
        """Gets the name of this ResponseWorkspaceList.  # noqa: E501


        :return: The name of this ResponseWorkspaceList.  # noqa: E501
        :rtype: str
        """
        return self._name

    @name.setter
    def name(self, name):
        """Sets the name of this ResponseWorkspaceList.


        :param name: The name of this ResponseWorkspaceList.  # noqa: E501
        :type name: str
        """
        if self.local_vars_configuration.client_side_validation and name is None:  # noqa: E501
            raise ValueError("Invalid value for `name`, must not be `None`")  # noqa: E501

        self._name = name

    @property
    def status(self):
        """Gets the status of this ResponseWorkspaceList.  # noqa: E501


        :return: The status of this ResponseWorkspaceList.  # noqa: E501
        :rtype: str
        """
        return self._status

    @status.setter
    def status(self, status):
        """Sets the status of this ResponseWorkspaceList.


        :param status: The status of this ResponseWorkspaceList.  # noqa: E501
        :type status: str
        """
        if self.local_vars_configuration.client_side_validation and status is None:  # noqa: E501
            raise ValueError("Invalid value for `status`, must not be `None`")  # noqa: E501

        self._status = status

    @property
    def status_last_updated(self):
        """Gets the status_last_updated of this ResponseWorkspaceList.  # noqa: E501


        :return: The status_last_updated of this ResponseWorkspaceList.  # noqa: E501
        :rtype: datetime
        """
        return self._status_last_updated

    @status_last_updated.setter
    def status_last_updated(self, status_last_updated):
        """Sets the status_last_updated of this ResponseWorkspaceList.


        :param status_last_updated: The status_last_updated of this ResponseWorkspaceList.  # noqa: E501
        :type status_last_updated: datetime
        """
        if self.local_vars_configuration.client_side_validation and status_last_updated is None:  # noqa: E501
            raise ValueError("Invalid value for `status_last_updated`, must not be `None`")  # noqa: E501

        self._status_last_updated = status_last_updated

    @property
    def status_reason(self):
        """Gets the status_reason of this ResponseWorkspaceList.  # noqa: E501


        :return: The status_reason of this ResponseWorkspaceList.  # noqa: E501
        :rtype: str
        """
        return self._status_reason

    @status_reason.setter
    def status_reason(self, status_reason):
        """Sets the status_reason of this ResponseWorkspaceList.


        :param status_reason: The status_reason of this ResponseWorkspaceList.  # noqa: E501
        :type status_reason: str
        """
        if self.local_vars_configuration.client_side_validation and status_reason is None:  # noqa: E501
            raise ValueError("Invalid value for `status_reason`, must not be `None`")  # noqa: E501

        self._status_reason = status_reason

    @property
    def updated_dt(self):
        """Gets the updated_dt of this ResponseWorkspaceList.  # noqa: E501


        :return: The updated_dt of this ResponseWorkspaceList.  # noqa: E501
        :rtype: datetime
        """
        return self._updated_dt

    @updated_dt.setter
    def updated_dt(self, updated_dt):
        """Sets the updated_dt of this ResponseWorkspaceList.


        :param updated_dt: The updated_dt of this ResponseWorkspaceList.  # noqa: E501
        :type updated_dt: datetime
        """
        if self.local_vars_configuration.client_side_validation and updated_dt is None:  # noqa: E501
            raise ValueError("Invalid value for `updated_dt`, must not be `None`")  # noqa: E501

        self._updated_dt = updated_dt

    def to_dict(self, serialize=False):
        """Returns the model properties as a dict"""
        result = {}

        def convert(x):
            if hasattr(x, "to_dict"):
                args = getfullargspec(x.to_dict).args
                if len(args) == 1:
                    return x.to_dict()
                else:
                    return x.to_dict(serialize)
            else:
                return x

        for attr, _ in six.iteritems(self.openapi_types):
            value = getattr(self, attr)
            attr = self.attribute_map.get(attr, attr) if serialize else attr
            if isinstance(value, list):
                result[attr] = list(map(
                    lambda x: convert(x),
                    value
                ))
            elif isinstance(value, dict):
                result[attr] = dict(map(
                    lambda item: (item[0], convert(item[1])),
                    value.items()
                ))
            else:
                result[attr] = convert(value)

        return result

    def to_str(self):
        """Returns the string representation of the model"""
        return pprint.pformat(self.to_dict())

    def __repr__(self):
        """For `print` and `pprint`"""
        return self.to_str()

    def __eq__(self, other):
        """Returns true if both objects are equal"""
        if not isinstance(other, ResponseWorkspaceList):
            return False

        return self.to_dict() == other.to_dict()

    def __ne__(self, other):
        """Returns true if both objects are not equal"""
        if not isinstance(other, ResponseWorkspaceList):
            return True

        return self.to_dict() != other.to_dict()



import sys

import click

import savvihub
from savvihub.cli._base import VesslGroup, vessl_argument, vessl_option
from savvihub.cli._util import (
    Endpoint,
    choices_prompter,
    format_bool,
    format_string,
    generic_prompter,
    print_data,
    print_table,
    prompt_choices,
)
from savvihub.cli.organization import organization_name_option
from savvihub.project import clone_project, create_project, list_projects, read_project
from savvihub.util.constant import (
    PROJECT_TYPE_VERSION_CONTROL,
    PROJECT_TYPES,
)
from savvihub.util.exception import InvalidProjectError


def project_name_prompter(
    ctx: click.Context, param: click.Parameter, value: str,
) -> str:
    projects = list_projects()
    return prompt_choices("Project", [x.name for x in projects])


def version_control_project_name_prompter(
    ctx: click.Context, param: click.Parameter, value: str,
) -> str:
    projects = [p for p in list_projects() if p.type == PROJECT_TYPE_VERSION_CONTROL]
    if not projects:
        raise click.ClickException(f'No version-control project found.')
    return prompt_choices("Project", [x.name for x in projects])


def github_owner_prompter(
    ctx: click.Context, param: click.Parameter, value: str,
) -> str:
    type = ctx.params.get("type")
    if type == PROJECT_TYPE_VERSION_CONTROL:
        return click.prompt("Github owner", type=click.STRING)


def github_repo_prompter(ctx: click.Context, param: click.Parameter, value: str) -> str:
    type = ctx.params.get("type")
    if type == PROJECT_TYPE_VERSION_CONTROL:
        return click.prompt("Github repository name", type=click.STRING)


@click.command(name="project", cls=VesslGroup)
def cli():
    pass


@cli.vessl_command()
@vessl_argument(
    "name", type=click.STRING, required=True, prompter=project_name_prompter
)
@organization_name_option
def read(name: str):
    project = read_project(project_name=name)

    git_data = "-"
    if project.type == PROJECT_TYPE_VERSION_CONTROL:
        git_data = {
            "Owner": project.cached_git_owner_slug,
            "Repository": project.cached_git_repo_slug,
            "URL": project.cached_git_http_url_to_repo,
            "Provider": format_string(project.git_provider),
        }

    print_data(
        {
            "ID": project.id,
            "Name": project.name,
            "Type": project.type,
            "Experiments": project.experiment_summary.total,
            "Public": format_bool(project.is_public),
            "Git": git_data,
        }
    )


@cli.vessl_command()
@organization_name_option
def list():
    projects = list_projects()
    print_table(
        projects,
        ["ID", "Name", "Type", "Experiments"],
        lambda x: [x.id, x.name, x.type, x.experiment_summary.total],
    )


@cli.vessl_command()
@vessl_argument(
    "type",
    type=click.Choice(PROJECT_TYPES),
    required=True,
    prompter=choices_prompter("Project type", PROJECT_TYPES),
)
@vessl_argument(
    "name", type=click.STRING, required=True, prompter=generic_prompter("Project name")
)
@vessl_option(
    "--github-owner",
    type=click.STRING,
    prompter=github_owner_prompter,
    help="Owner of github repository.",
)
@vessl_option("--github-repo", type=click.STRING, prompter=github_repo_prompter)
@click.option("-m", "--description", type=click.STRING)
@organization_name_option
def create(type: str, name: str, github_owner: str, github_repo: str, description: str):
    project = create_project(
        project_type=type,
        project_name=name,
        github_owner=github_owner,
        github_repo=github_repo,
        description=description,
    )
    print(
        f"Created '{project.name}'.\n"
        f"For more info: {Endpoint.project.format(project.organization.name, project.name)}"
    )


@cli.vessl_command()
@vessl_argument(
    "name", type=click.STRING, required=True, prompter=version_control_project_name_prompter,
)
@organization_name_option
def clone(name: str):
    clone_project(project_name=name)


def project_name_callback(
    ctx: click.Context, param: click.Parameter, value: str,
):
    if savvihub.vessl_api.organization is None:
        savvihub.vessl_api.set_organization()

    try:
        savvihub.vessl_api.set_project(value)
    except InvalidProjectError:
        print("Invalid project. Please choose a project using `savvihub configure`.")
        sys.exit(1)


# Ensure this is called before other options with `is_eager=True` for
# other callbacks that need organization to be preconfigured.
project_name_option = click.option(
    "--project",
    "project_name",
    type=click.STRING,
    hidden=True,
    is_eager=True,
    expose_value=False,
    callback=project_name_callback,
    help="Override default project.",
)



import os
from collections import defaultdict
from datetime import datetime
from numbers import Number
from typing import Any, Dict, List, Union

from openapi_client.models import (
    InfluxdbExperimentPlotFile,
    InfluxdbExperimentPlotMetric,
    ResponseExperimentInfo,
)
from savvihub import vessl_api
from savvihub.experiment import (
    read_experiment_by_id,
    update_experiment_plots_files,
    update_experiment_plots_metrics,
)
from savvihub.util import logger
from savvihub.util.constant import VESSL_IMAGE_PATH, VESSL_PLOTS_FILETYPE_IMAGE
from savvihub.util.exception import SavvihubApiException
from savvihub.util.image import Image
from savvihub.volume import copy_volume_file

ImageRowType = Dict[str, List[Image]]
MetricRowType = Dict[str, Number]
RowType = Union[ImageRowType, MetricRowType]

current_experiment: ResponseExperimentInfo = None


def _update_current_experiment():
    global current_experiment
    experiment_id = os.environ.get(
        "SAVVIHUB_EXPERIMENT_ID", None
    )  # TODO: rename on server
    access_token = os.environ.get(
        "SAVVIHUB_ACCESS_TOKEN", None
    )  # TODO: rename on server

    if experiment_id is None or access_token is None:
        return

    vessl_api.update_access_token(access_token)
    current_experiment = read_experiment_by_id(experiment_id)


def get_current_experiment() -> ResponseExperimentInfo:
    global current_experiment
    if current_experiment != None:
        return current_experiment

    _update_current_experiment()
    return current_experiment


def _update_images(row: ImageRowType):
    path_to_caption = {}
    for images in row.values():
        for image in images:
            path_to_caption[image.path] = image.caption

    source_path = VESSL_IMAGE_PATH + "/"
    dest_volume_id = current_experiment.experiment_plot_volume
    dest_path = "/"

    files = copy_volume_file(
        source_volume_id=None,
        source_path=source_path,
        dest_volume_id=dest_volume_id,
        dest_path=dest_path,
        recursive=True,
    )

    for image in images:
        image.flush()

    plot_files = [
        InfluxdbExperimentPlotFile(
            step=None,
            path=file.path,
            caption=path_to_caption[file.path],
            timestamp=datetime.utcnow().timestamp(),
        )
        for file in files
        if file.path in path_to_caption
    ]

    update_experiment_plots_files(
        experiment_id=current_experiment.id,
        files=plot_files,
        type=VESSL_PLOTS_FILETYPE_IMAGE,
    )


def _update_metrics(row: MetricRowType, step: int):
    plot_metrics: Dict[str, List[InfluxdbExperimentPlotMetric]] = defaultdict(list)

    for key, val in row.items():
        plot_metrics[key].append(
            InfluxdbExperimentPlotMetric(
                step=step,
                timestamp=datetime.utcnow().timestamp(),
                value=float(val),
            )
        )

    workload_id = int(os.environ.get(
        "SAVVIHUB_WORKLOAD_ID", None
    ))  # TODO: rename on server

    update_experiment_plots_metrics(
        experiment_id=current_experiment.id, workload_id=workload_id, metrics=plot_metrics
    )


def _log(row: RowType, step: int = None):
    # TODO: type validation? (ref: legacy client)
    # row, step = _refine(row, step)

    for val in row.values():
        if isinstance(val, list) and all(isinstance(i, Image) for i in val):
            _update_images(row)
        else:
            _update_metrics(row, step)
        break


def log(row: RowType, step: int = None):
    """Log a metric during a Vessl experiment.

    This function must be called on the Vessl infrastructure to log the metric.
    If not executed on Vessl's infrastructure, this function has no effect.

    :param row: a dictionary to log (required)
    :param step: a step(positive integer) for each iteration
    """

    _update_current_experiment()

    try:
        _log(row, step)

    except SavvihubApiException as e:
        logger.warn(f"Cannot send metrics {row} for step {step}: {e.message}")



from test.conftest import USE_MOCK

import pytest

import savvihub
from savvihub.util.constant import (
    PROJECT_TYPE_CLI_DRIVEN,
    SWEEP_ALGORITHM_TYPE_RANDOM,
    SWEEP_OBJECTIVE_TYPE_MAXIMIZE,
)


@pytest.mark.skipif(USE_MOCK, reason="Does not run if mocking is used.")
class TestSweep:
    @pytest.mark.order(index=1)
    def test_create_sweep(self):
        # Requires sweep suggestion server
        # pytest.sweep = savvihub.create_sweep(
        #     objective_type=SWEEP_OBJECTIVE_TYPE_MAXIMIZE,
        #     objective_goal=0.99,
        #     objective_metric="val_accuracy",
        #     max_experiment_count=2,
        #     parallel_experiment_count=1,
        #     max_failed_experiment_count=1,
        #     algorithm=SWEEP_ALGORITHM_TYPE_RANDOM,
        #     parameters=[
        #         {"name": "a", "type": "int", "range": {"list": ["1", "2", "3"]}}
        #     ],
        #     cluster_name="local",
        #     start_command="echo 'Hello world!'",
        #     kernel_resource_spec_name="v1.cpu-0.mem-1",
        #     kernel_image_url="savvihub/kernels:py36.full-cpu",
        #     local_project_url="",
        # )
        pass

    def test_read_sweep(self):
        # Requires sweep suggestion server
        pass

    def test_list_sweeps(self):
        savvihub.list_sweeps()

    def test_terminate_sweep(self):
        # Requires sweep suggestion server
        pass

    def test_list_sweep_logs(self):
        # Requires sweep suggestion server
        pass



# coding: utf-8

"""
    Aron API

    No description provided (generated by Openapi Generator https://github.com/openapitools/openapi-generator)  # noqa: E501

    The version of the OpenAPI document: 1.0.0
    Generated by: https://openapi-generator.tech
"""


try:
    from inspect import getfullargspec
except ImportError:
    from inspect import getargspec as getfullargspec
import pprint
import re  # noqa: F401
import six

from openapi_client.configuration import Configuration


class OrmFileMetadata(object):
    """NOTE: This class is auto generated by OpenAPI Generator.
    Ref: https://openapi-generator.tech

    Do not edit the class manually.
    """

    """
    Attributes:
      openapi_types (dict): The key is attribute name
                            and the value is attribute type.
      attribute_map (dict): The key is attribute name
                            and the value is json key in definition.
    """
    openapi_types = {
        'h': 'str',
        'l': 'str',
        'p': 'str',
        's': 'int',
        't': 'str',
        'u': 'datetime',
        'v': 'int'
    }

    attribute_map = {
        'h': 'h',
        'l': 'l',
        'p': 'p',
        's': 's',
        't': 't',
        'u': 'u',
        'v': 'v'
    }

    def __init__(self, h=None, l=None, p=None, s=None, t=None, u=None, v=None, local_vars_configuration=None):  # noqa: E501
        """OrmFileMetadata - a model defined in OpenAPI"""  # noqa: E501
        if local_vars_configuration is None:
            local_vars_configuration = Configuration.get_default_copy()
        self.local_vars_configuration = local_vars_configuration

        self._h = None
        self._l = None
        self._p = None
        self._s = None
        self._t = None
        self._u = None
        self._v = None
        self.discriminator = None

        if h is not None:
            self.h = h
        if l is not None:
            self.l = l
        if p is not None:
            self.p = p
        if s is not None:
            self.s = s
        if t is not None:
            self.t = t
        if u is not None:
            self.u = u
        if v is not None:
            self.v = v

    @property
    def h(self):
        """Gets the h of this OrmFileMetadata.  # noqa: E501


        :return: The h of this OrmFileMetadata.  # noqa: E501
        :rtype: str
        """
        return self._h

    @h.setter
    def h(self, h):
        """Sets the h of this OrmFileMetadata.


        :param h: The h of this OrmFileMetadata.  # noqa: E501
        :type h: str
        """

        self._h = h

    @property
    def l(self):
        """Gets the l of this OrmFileMetadata.  # noqa: E501


        :return: The l of this OrmFileMetadata.  # noqa: E501
        :rtype: str
        """
        return self._l

    @l.setter
    def l(self, l):
        """Sets the l of this OrmFileMetadata.


        :param l: The l of this OrmFileMetadata.  # noqa: E501
        :type l: str
        """

        self._l = l

    @property
    def p(self):
        """Gets the p of this OrmFileMetadata.  # noqa: E501


        :return: The p of this OrmFileMetadata.  # noqa: E501
        :rtype: str
        """
        return self._p

    @p.setter
    def p(self, p):
        """Sets the p of this OrmFileMetadata.


        :param p: The p of this OrmFileMetadata.  # noqa: E501
        :type p: str
        """

        self._p = p

    @property
    def s(self):
        """Gets the s of this OrmFileMetadata.  # noqa: E501


        :return: The s of this OrmFileMetadata.  # noqa: E501
        :rtype: int
        """
        return self._s

    @s.setter
    def s(self, s):
        """Sets the s of this OrmFileMetadata.


        :param s: The s of this OrmFileMetadata.  # noqa: E501
        :type s: int
        """

        self._s = s

    @property
    def t(self):
        """Gets the t of this OrmFileMetadata.  # noqa: E501


        :return: The t of this OrmFileMetadata.  # noqa: E501
        :rtype: str
        """
        return self._t

    @t.setter
    def t(self, t):
        """Sets the t of this OrmFileMetadata.


        :param t: The t of this OrmFileMetadata.  # noqa: E501
        :type t: str
        """

        self._t = t

    @property
    def u(self):
        """Gets the u of this OrmFileMetadata.  # noqa: E501


        :return: The u of this OrmFileMetadata.  # noqa: E501
        :rtype: datetime
        """
        return self._u

    @u.setter
    def u(self, u):
        """Sets the u of this OrmFileMetadata.


        :param u: The u of this OrmFileMetadata.  # noqa: E501
        :type u: datetime
        """

        self._u = u

    @property
    def v(self):
        """Gets the v of this OrmFileMetadata.  # noqa: E501


        :return: The v of this OrmFileMetadata.  # noqa: E501
        :rtype: int
        """
        return self._v

    @v.setter
    def v(self, v):
        """Sets the v of this OrmFileMetadata.


        :param v: The v of this OrmFileMetadata.  # noqa: E501
        :type v: int
        """

        self._v = v

    def to_dict(self, serialize=False):
        """Returns the model properties as a dict"""
        result = {}

        def convert(x):
            if hasattr(x, "to_dict"):
                args = getfullargspec(x.to_dict).args
                if len(args) == 1:
                    return x.to_dict()
                else:
                    return x.to_dict(serialize)
            else:
                return x

        for attr, _ in six.iteritems(self.openapi_types):
            value = getattr(self, attr)
            attr = self.attribute_map.get(attr, attr) if serialize else attr
            if isinstance(value, list):
                result[attr] = list(map(
                    lambda x: convert(x),
                    value
                ))
            elif isinstance(value, dict):
                result[attr] = dict(map(
                    lambda item: (item[0], convert(item[1])),
                    value.items()
                ))
            else:
                result[attr] = convert(value)

        return result

    def to_str(self):
        """Returns the string representation of the model"""
        return pprint.pformat(self.to_dict())

    def __repr__(self):
        """For `print` and `pprint`"""
        return self.to_str()

    def __eq__(self, other):
        """Returns true if both objects are equal"""
        if not isinstance(other, OrmFileMetadata):
            return False

        return self.to_dict() == other.to_dict()

    def __ne__(self, other):
        """Returns true if both objects are not equal"""
        if not isinstance(other, OrmFileMetadata):
            return True

        return self.to_dict() != other.to_dict()



import os
from typing import List

from openapi_client.models import ResponseSSHKeyInfo
from openapi_client.models.ssh_key_create_api_payload import SSHKeyCreateAPIPayload
from savvihub import vessl_api


def list_ssh_keys() -> List[ResponseSSHKeyInfo]:
    return vessl_api.s_sh_key_list_api().ssh_keys


def create_ssh_key(key_path: str, key_name: str, ssh_public_key_value: str) -> ResponseSSHKeyInfo:
    return vessl_api.s_sh_key_create_api(
        ssh_key_create_api_payload=SSHKeyCreateAPIPayload(
            filename=os.path.basename(key_path),
            name=key_name,
            public_key=ssh_public_key_value,
        )
    )


def delete_ssh_key(key_id: int) -> object:
    return vessl_api.s_sh_key_delete_api(ssh_key_id=key_id)



import os
from pathlib import Path
from typing import Optional

import click

from savvihub import list_ssh_keys
from savvihub.cli._base import VesslGroup, vessl_option
from savvihub.cli._util import (
    prompt_choices,
)
from savvihub.cli.organization import organization_name_option
from savvihub.util.constant import SSH_CONFIG_PATH
from savvihub.workspace import (
    backup_workspace,
    connect_workspace_ssh,
    list_workspaces,
    restore_workspace,
    update_vscode_remote_ssh,
)


def ssh_private_key_path_callback(ctx: click.Context, param: click.Parameter, key_path: str) -> Optional[str]:
    if key_path:
        return key_path

    ssh_keys = list_ssh_keys()
    if len(ssh_keys) == 0:
        raise click.BadParameter('At least one ssh public key should be added.\n'
                                 'Please run `savvihub ssh keys add`.')

    home = Path.home()
    for ssh_key in ssh_keys:
        key_path = f'{home}/.ssh/{ssh_key.filename.rstrip(".pub")}'
        if os.path.exists(key_path):
            return key_path

    for key_path in (f'{home}/.ssh/id_rsa', f'{home}/.ssh/id_ed25519'):
        if os.path.exists(key_path):
            return key_path

    return None


def workspace_id_prompter(
    ctx: click.Context, param: click.Parameter, value: int
) -> int:
    workspaces = list_workspaces()
    return prompt_choices("Workspace", [(x.name, x.id) for x in workspaces])


@click.command(name="workspace", cls=VesslGroup)
def cli():
    pass


@cli.vessl_command()
@vessl_option(
    "-p",
    "--key-path",
    type=click.Path(exists=True),
    callback=ssh_private_key_path_callback,
    help="Path to SSH private key.",
)
@organization_name_option
def ssh(key_path: str):
    connect_workspace_ssh(private_key_path=key_path)


@cli.vessl_command()
@vessl_option(
    "-p",
    "--key-path",
    type=click.STRING,
    callback=ssh_private_key_path_callback,
    help='SSH private key path.',
)
@organization_name_option
def vscode(key_path: str):
    update_vscode_remote_ssh(private_key_path=key_path)
    print(f"Updated '{SSH_CONFIG_PATH}'.")


@cli.vessl_command()
@organization_name_option
def backup():
    backup_workspace()


@cli.vessl_command()
@organization_name_option
def restore():
    restore_workspace()



from test.conftest import USE_MOCK
from test.util import random_string

import pytest

import savvihub


@pytest.mark.skipif(USE_MOCK, reason="Does not run if mocking is used.")
class TestVolume:
    file_name = random_string()

    @pytest.mark.order(index=1)
    def test_create_volume_file(self):
        volume_id = savvihub.vessl_api.project.volume_id
        savvihub.create_volume_file(volume_id, False, self.file_name)

    def test_read_volume_file(self):
        volume_id = savvihub.vessl_api.project.volume_id
        savvihub.read_volume_file(volume_id, self.file_name)

    def test_list_volume_files(self):
        volume_id = savvihub.vessl_api.project.volume_id
        savvihub.list_volume_files(volume_id)

    @pytest.mark.order(index=-1)
    def test_delete_volume_file(self):
        volume_id = savvihub.vessl_api.project.volume_id
        savvihub.delete_volume_file(volume_id, path=self.file_name)



import click

from savvihub.cli._base import VesslGroup, vessl_argument
from savvihub.cli._util import (
    format_string,
    generic_prompter,
    print_data,
    print_table,
    prompt_choices,
)
from savvihub.cli.organization import organization_name_option
from savvihub.kernel_cluster import (
    delete_cluster,
    list_cluster_nodes,
    list_clusters,
    read_cluster,
    rename_cluster,
)


def cluster_name_prompter(
    ctx: click.Context, param: click.Parameter, value: str
) -> str:
    clusters = list_clusters()
    return prompt_choices("Cluster", [x.name for x in clusters])


def custom_cluster_name_prompter(
    ctx: click.Context, param: click.Parameter, value: str
) -> str:
    clusters = list_clusters()
    clusters = [x for x in clusters if not x.is_savvihub_managed]
    return prompt_choices("Cluster", [x.name for x in clusters])


@click.command(name="cluster", cls=VesslGroup)
def cli():
    pass


@cli.vessl_command()
@vessl_argument(
    "name", type=click.STRING, required=True, prompter=cluster_name_prompter
)
@organization_name_option
def read(name: str):
    cluster = read_cluster(cluster_name=name)
    print_data(
        {
            "ID": cluster.id,
            "Name": cluster.name,
            "Type": "Managed" if cluster.is_savvihub_managed else "Custom",
            "Region": format_string(cluster.region),
            "Status": cluster.status.replace("-", " "),
            "K8s Master Endpoint": format_string(cluster.name),
            "K8s Namespace": format_string(cluster.kubernetes_namespace),
            "K8s Service Type": cluster.kubernetes_service_type,
        }
    )


@cli.vessl_command()
@organization_name_option
def list():
    clusters = list_clusters()
    print_table(
        clusters,
        ["ID", "Name", "Type", "Status", "K8s Master Endpoint", "K8s Namespace"],
        lambda x: [
            x.id,
            x.name,
            "Managed" if x.is_savvihub_managed else "Custom",
            x.status.replace("-", " "),
            format_string(x.name),
            format_string(x.kubernetes_namespace),
        ],
    )


@cli.vessl_command()
@vessl_argument(
    "name", type=click.STRING, required=True, prompter=custom_cluster_name_prompter
)
@organization_name_option
def delete(name: str):
    click.confirm(f"Are you sure you want to delete '{name}'?", abort=True)
    data = delete_cluster(cluster_name=name)
    print(f"Deleted '{name}'.")


@cli.vessl_command()
@vessl_argument(
    "name", type=click.STRING, required=True, prompter=custom_cluster_name_prompter
)
@vessl_argument(
    "new_name", type=click.STRING, required=True, prompter=generic_prompter("New name")
)
@organization_name_option
def rename(name: str, new_name: str):
    cluster = rename_cluster(cluster_name=name, new_cluster_name=new_name)
    print(f"Renamed '{name}' to '{cluster.name}'.")


@cli.vessl_command()
@vessl_argument(
    "name", type=click.STRING, required=True, prompter=custom_cluster_name_prompter
)
@organization_name_option
def list_nodes(name: str):
    cluster_nodes = list_cluster_nodes(cluster_name=name)
    print_table(
        cluster_nodes,
        ["ID", "Name", "CPU Limits", "GPU Limits", "Memory Limits"],
        lambda x: [x.id, x.name, x.cpu_limits, x.gpu_limits, x.memory_limits],
    )



# coding: utf-8

"""
    Aron API

    No description provided (generated by Openapi Generator https://github.com/openapitools/openapi-generator)  # noqa: E501

    The version of the OpenAPI document: 1.0.0
    Generated by: https://openapi-generator.tech
"""


try:
    from inspect import getfullargspec
except ImportError:
    from inspect import getargspec as getfullargspec
import pprint
import re  # noqa: F401
import six

from openapi_client.configuration import Configuration


class OrmProjectEdges(object):
    """NOTE: This class is auto generated by OpenAPI Generator.
    Ref: https://openapi-generator.tech

    Do not edit the class manually.
    """

    """
    Attributes:
      openapi_types (dict): The key is attribute name
                            and the value is attribute type.
      attribute_map (dict): The key is attribute name
                            and the value is json key in definition.
    """
    openapi_types = {
        'experiments': 'list[OrmExperiment]',
        'models': 'list[OrmModel]',
        'organization': 'OrmOrganization',
        'primary_owner': 'OrmUser',
        'repositories': 'list[OrmProjectRepository]',
        'tags': 'list[OrmTag]',
        'volume': 'OrmVolume'
    }

    attribute_map = {
        'experiments': 'experiments',
        'models': 'models',
        'organization': 'organization',
        'primary_owner': 'primary_owner',
        'repositories': 'repositories',
        'tags': 'tags',
        'volume': 'volume'
    }

    def __init__(self, experiments=None, models=None, organization=None, primary_owner=None, repositories=None, tags=None, volume=None, local_vars_configuration=None):  # noqa: E501
        """OrmProjectEdges - a model defined in OpenAPI"""  # noqa: E501
        if local_vars_configuration is None:
            local_vars_configuration = Configuration.get_default_copy()
        self.local_vars_configuration = local_vars_configuration

        self._experiments = None
        self._models = None
        self._organization = None
        self._primary_owner = None
        self._repositories = None
        self._tags = None
        self._volume = None
        self.discriminator = None

        if experiments is not None:
            self.experiments = experiments
        if models is not None:
            self.models = models
        if organization is not None:
            self.organization = organization
        if primary_owner is not None:
            self.primary_owner = primary_owner
        if repositories is not None:
            self.repositories = repositories
        if tags is not None:
            self.tags = tags
        if volume is not None:
            self.volume = volume

    @property
    def experiments(self):
        """Gets the experiments of this OrmProjectEdges.  # noqa: E501


        :return: The experiments of this OrmProjectEdges.  # noqa: E501
        :rtype: list[OrmExperiment]
        """
        return self._experiments

    @experiments.setter
    def experiments(self, experiments):
        """Sets the experiments of this OrmProjectEdges.


        :param experiments: The experiments of this OrmProjectEdges.  # noqa: E501
        :type experiments: list[OrmExperiment]
        """

        self._experiments = experiments

    @property
    def models(self):
        """Gets the models of this OrmProjectEdges.  # noqa: E501


        :return: The models of this OrmProjectEdges.  # noqa: E501
        :rtype: list[OrmModel]
        """
        return self._models

    @models.setter
    def models(self, models):
        """Sets the models of this OrmProjectEdges.


        :param models: The models of this OrmProjectEdges.  # noqa: E501
        :type models: list[OrmModel]
        """

        self._models = models

    @property
    def organization(self):
        """Gets the organization of this OrmProjectEdges.  # noqa: E501


        :return: The organization of this OrmProjectEdges.  # noqa: E501
        :rtype: OrmOrganization
        """
        return self._organization

    @organization.setter
    def organization(self, organization):
        """Sets the organization of this OrmProjectEdges.


        :param organization: The organization of this OrmProjectEdges.  # noqa: E501
        :type organization: OrmOrganization
        """

        self._organization = organization

    @property
    def primary_owner(self):
        """Gets the primary_owner of this OrmProjectEdges.  # noqa: E501


        :return: The primary_owner of this OrmProjectEdges.  # noqa: E501
        :rtype: OrmUser
        """
        return self._primary_owner

    @primary_owner.setter
    def primary_owner(self, primary_owner):
        """Sets the primary_owner of this OrmProjectEdges.


        :param primary_owner: The primary_owner of this OrmProjectEdges.  # noqa: E501
        :type primary_owner: OrmUser
        """

        self._primary_owner = primary_owner

    @property
    def repositories(self):
        """Gets the repositories of this OrmProjectEdges.  # noqa: E501


        :return: The repositories of this OrmProjectEdges.  # noqa: E501
        :rtype: list[OrmProjectRepository]
        """
        return self._repositories

    @repositories.setter
    def repositories(self, repositories):
        """Sets the repositories of this OrmProjectEdges.


        :param repositories: The repositories of this OrmProjectEdges.  # noqa: E501
        :type repositories: list[OrmProjectRepository]
        """

        self._repositories = repositories

    @property
    def tags(self):
        """Gets the tags of this OrmProjectEdges.  # noqa: E501


        :return: The tags of this OrmProjectEdges.  # noqa: E501
        :rtype: list[OrmTag]
        """
        return self._tags

    @tags.setter
    def tags(self, tags):
        """Sets the tags of this OrmProjectEdges.


        :param tags: The tags of this OrmProjectEdges.  # noqa: E501
        :type tags: list[OrmTag]
        """

        self._tags = tags

    @property
    def volume(self):
        """Gets the volume of this OrmProjectEdges.  # noqa: E501


        :return: The volume of this OrmProjectEdges.  # noqa: E501
        :rtype: OrmVolume
        """
        return self._volume

    @volume.setter
    def volume(self, volume):
        """Sets the volume of this OrmProjectEdges.


        :param volume: The volume of this OrmProjectEdges.  # noqa: E501
        :type volume: OrmVolume
        """

        self._volume = volume

    def to_dict(self, serialize=False):
        """Returns the model properties as a dict"""
        result = {}

        def convert(x):
            if hasattr(x, "to_dict"):
                args = getfullargspec(x.to_dict).args
                if len(args) == 1:
                    return x.to_dict()
                else:
                    return x.to_dict(serialize)
            else:
                return x

        for attr, _ in six.iteritems(self.openapi_types):
            value = getattr(self, attr)
            attr = self.attribute_map.get(attr, attr) if serialize else attr
            if isinstance(value, list):
                result[attr] = list(map(
                    lambda x: convert(x),
                    value
                ))
            elif isinstance(value, dict):
                result[attr] = dict(map(
                    lambda item: (item[0], convert(item[1])),
                    value.items()
                ))
            else:
                result[attr] = convert(value)

        return result

    def to_str(self):
        """Returns the string representation of the model"""
        return pprint.pformat(self.to_dict())

    def __repr__(self):
        """For `print` and `pprint`"""
        return self.to_str()

    def __eq__(self, other):
        """Returns true if both objects are equal"""
        if not isinstance(other, OrmProjectEdges):
            return False

        return self.to_dict() == other.to_dict()

    def __ne__(self, other):
        """Returns true if both objects are not equal"""
        if not isinstance(other, OrmProjectEdges):
            return True

        return self.to_dict() != other.to_dict()



import sys

import click

import savvihub
from savvihub.cli._base import VesslGroup, vessl_argument
from savvihub.cli._util import (
    Endpoint,
    generic_prompter,
    print_data,
    print_table,
    prompt_choices,
)
from savvihub.organization import (
    create_organization,
    list_organizations,
    read_organization,
)
from savvihub.util.exception import InvalidOrganizationError


def organization_name_prompter(
    ctx: click.Context, param: click.Parameter, value: str
) -> str:
    organizations = list_organizations()
    return prompt_choices("Organization", [x.name for x in organizations])


def region_prompter(ctx: click.Context, param: click.Parameter, value: str) -> str:
    regions = savvihub.vessl_api.region_list_api().regions
    return prompt_choices("Region", [(x.name, x.value) for x in regions])


@click.command(name="organization", cls=VesslGroup)
def cli():
    pass


@cli.vessl_command()
@vessl_argument(
    "name",
    type=click.STRING,
    required=True,
    prompter=organization_name_prompter,
)
def read(name: str):
    organization = read_organization(organization_name=name)
    print_data(
        {
            "ID": organization.id,
            "Name": organization.name,
            "Region": organization.default_region,
            "Description": organization.description,
        }
    )


@cli.vessl_command()
def list():
    organizations = list_organizations()
    print_table(
        organizations,
        ["ID", "Name", "Region"],
        lambda x: [x.id, x.name, x.default_region],
    )


@cli.vessl_command()
@vessl_argument(
    "name",
    type=click.STRING,
    required=True,
    prompter=generic_prompter("Organization name"),
)
@vessl_argument("region", type=click.STRING, required=True, prompter=region_prompter)
def create(name: str, region: str):
    organization = create_organization(organization_name=name, region=region)
    print(
        f"Created '{organization.name}'.\n"
        f"For more info: {Endpoint.organization.format(organization.name)}"
    )


def organization_name_callback(
    ctx: click.Context, param: click.Parameter, value: str
) -> str:
    try:
        savvihub.vessl_api.set_organization(value)
    except InvalidOrganizationError:
        print(
            "Invalid organization. Please choose an organization using `savvihub configure`."
        )
        sys.exit(1)


# Ensure this is called before other options with `is_eager=True` for
# other callbacks that need organization to be preconfigured.
organization_name_option = click.option(
    "--organization",
    "organization_name",
    type=click.STRING,
    hidden=True,
    is_eager=True,
    expose_value=False,
    callback=organization_name_callback,
    help="Override default organization.",
)



import os
from typing import List, Optional, Tuple

from openapi_client.models import (
    ProtoVolumeMountRequest,
    ProtoVolumeMountRequests,
    ProtoVolumeMountRequestSourceCLIDrivenProject,
    ProtoVolumeMountRequestSourceDataset,
    ProtoVolumeMountRequestSourceDatasetVersion,
    ProtoVolumeMountRequestSourceProject,
    ProtoVolumeMountRequestSourceVersionControlProject,
    ResponseFileMetadata,
    VolumeFileCopyAPIPayload,
    VolumeFileCreateAPIPayload,
)
from savvihub import vessl_api
from savvihub.organization import _get_organization_name
from savvihub.project import _get_project
from savvihub.util.constant import (
    DATASET_VERSION_HASH_LATEST,
    MOUNT_PATH_EMPTY_DIR,
    MOUNT_PATH_PROJECT,
    MOUNT_TYPE_DATASET,
    MOUNT_TYPE_DATSET_VERSION,
    MOUNT_TYPE_EMPTY_DIR,
    MOUNT_TYPE_OUTPUT,
    MOUNT_TYPE_PROJECT,
    PROJECT_TYPE_CLI_DRIVEN,
    PROJECT_TYPE_VERSION_CONTROL,
)
from savvihub.util.downloader import Downloader
from savvihub.util.exception import InvalidVolumeFileError, SavvihubApiException
from savvihub.util.git import get_git_branch, get_git_diff_path, get_git_ref
from savvihub.util.uploader import Uploader
from savvihub.util.zipper import Zipper


def read_volume_file(volume_id: int, path: str) -> ResponseFileMetadata:
    return vessl_api.volume_file_read_api(volume_id=volume_id, path=path)


def list_volume_files(
    volume_id: int,
    need_download_url: bool = False,
    path: str = "",
    recursive: bool = False,
) -> List[ResponseFileMetadata]:
    return vessl_api.volume_file_list_api(
        volume_id=volume_id,
        recursive=recursive,
        path=path,
        need_download_url=need_download_url,
    ).results


def create_volume_file(volume_id: int, is_dir: bool, path: str) -> ResponseFileMetadata:
    return vessl_api.volume_file_create_api(
        volume_id=volume_id,
        volume_file_create_api_payload=VolumeFileCreateAPIPayload(
            is_dir=is_dir,
            path=path,
        ),
    )


def delete_volume_file(
    volume_id: int, path: str, recursive: bool = False
) -> List[ResponseFileMetadata]:
    return vessl_api.volume_file_delete_api(
        volume_id=volume_id, path=path, recursive=recursive
    ).deleted_files


def upload_volume_file(volume_id: int, path: str) -> ResponseFileMetadata:
    return vessl_api.volume_file_uploaded_api(volume_id=volume_id, path=path)


def copy_volume_file(
    source_volume_id: Optional[int],
    source_path: str,
    dest_volume_id: Optional[int],
    dest_path: str,
    recursive: bool = False,
) -> None:
    if source_volume_id is None:
        return _copy_volume_file_local_to_remote(
            source_path, dest_volume_id, dest_path, recursive
        )

    if dest_volume_id is None:
        return _copy_volume_file_remote_to_local(
            source_volume_id, source_path, dest_path, recursive
        )

    return _copy_volume_file_remote_to_remote(
        source_volume_id, source_path, dest_volume_id, dest_path, recursive
    )


def _copy_volume_file_local_to_remote(
    source_path: str, dest_volume_id: int, dest_path: str, recursive: bool
) -> None:
    """Copy local to remote

    Behavior works like linux cp command
    - `source_path` is file
      - `dest_path` is not a directory: copy as file with new name
      - `dest_path` is directory: copy file into directory with original name
    - `source_path` is directory
      - `dest_path` is file: error
      - `dest_path` does not exist: create `dest_path` and copy contents of `source_path`
      - `dest_path` exists: copy `source_path` as subdirectory of `dest_path`
    """

    output = "Successfully uploaded {} out of {} file(s)."
    source_path = source_path.rstrip("/")

    try:
        dest_file = read_volume_file(dest_volume_id, dest_path)
    except SavvihubApiException:
        dest_file = None

    if not os.path.isdir(source_path):
        if dest_file and (dest_file.is_dir or dest_path.endswith("/")):
            dest_path = os.path.join(dest_path, os.path.basename(source_path))

        uploaded_file = Uploader.upload(source_path, dest_volume_id, dest_path)

        print(output.format(1, 1))
        return uploaded_file

    if dest_file and not dest_file.is_dir:
        raise InvalidVolumeFileError(
            f"Destination path is not a directory: {dest_path}."
        )

    if dest_file and dest_file.is_dir:
        dest_path = os.path.join(dest_path, os.path.basename(source_path))

    paths = Uploader.get_paths_in_dir(source_path)
    uploaded_files = Uploader.bulk_upload(source_path, paths, dest_volume_id, dest_path)

    print(output.format(len(uploaded_files), len(paths)))
    return uploaded_files


def _copy_volume_file_remote_to_local(
    source_volume_id: int, source_path: str, dest_path: str, recursive: bool
) -> None:
    """Copy remote to local

    Behavior works like linux cp command
    - `source_path` is file
      - `dest_path` is not a directory: copy as file with new name
      - `dest_path` is directory: copy file into directory with original name
    - `source_path` is directory
      - `dest_path` is file: error
      - `dest_path` does not exist: create `dest_path` and copy contents of `source_path`
      - `dest_path` exists: copy `source_path` as subdirectory of `dest_path`
    """

    output = "Successfully downloaded {} out of {} file(s)."
    source_file = read_volume_file(source_volume_id, source_path)

    if not source_file.is_dir:
        if os.path.isdir(dest_path):
            dest_path = os.path.join(dest_path, os.path.basename(source_file.path))

        Downloader.download(dest_path, source_file)

        print(output.format(1, 1))
        return

    files = list_volume_files(
        volume_id=source_volume_id,
        need_download_url=True,
        path=source_path,
        recursive=True,
    )

    if os.path.isfile(dest_path):
        raise InvalidVolumeFileError(
            f"Destination path is not a directory: {dest_path}."
        )

    prefix = source_path.rstrip("/")
    if os.path.isdir(dest_path):
        prefix = os.path.dirname(prefix)

    if prefix:
        prefix += "/"
        for file in files:
            file.path = file.path.replace(prefix, "", 1)

    downloaded_files = Downloader.bulk_download(dest_path, files)
    print(output.format(len(downloaded_files), len([x for x in files if not x.is_dir])))


def _copy_volume_file_remote_to_remote(
    source_volume_id: int,
    source_path: str,
    dest_volume_id: int,
    dest_path: str,
    recursive: bool,
) -> None:
    if source_volume_id != dest_volume_id:
        raise InvalidVolumeFileError("Files can only be copied within the same volume.")

    files = vessl_api.volume_file_copy_api(
        volume_id=source_volume_id,
        volume_file_copy_api_payload=VolumeFileCopyAPIPayload(
            dest_path=dest_path,
            recursive=recursive,
            source_dataset_version="latest",
            source_path=source_path,
        ),
    ).copied_files

    print(f"Successfully downloaded {len(files)} out of {len(files)} file(s).")


def _configure_volume_mount_requests(
    dataset_mounts: List[Tuple[str, str]],
    root_volume_size: str,
    working_dir: str,
    output_dir: str,
    local_project_url: str,
    **kwargs,
) -> ProtoVolumeMountRequests:
    requests = [
        _configure_volume_mount_request_empty_dir(),
        _configure_volume_mount_request_output(output_dir),
        _configure_volume_mount_request_project(local_project_url, **kwargs),
    ]

    if dataset_mounts is not None:
        requests += [
            _configure_volume_mount_request_dataset(mount_path, dataset_name, **kwargs)
            for mount_path, dataset_name in dataset_mounts
        ]

    return ProtoVolumeMountRequests(
        root_volume_size=root_volume_size,
        working_dir=working_dir,
        requests=requests,
    )


def _configure_volume_mount_request_dataset(
    mount_path: str, dataset_name: str, **kwargs
) -> ProtoVolumeMountRequest:
    from savvihub.dataset import read_dataset, read_dataset_version

    mount_path = os.path.join(mount_path, "")  # Ensure path ends in /

    organization_name = _get_organization_name(**kwargs)
    dataset_version_hash = DATASET_VERSION_HASH_LATEST

    if "@" in dataset_name:
        # Example: mnist@3d1e0f91c
        dataset_name, dataset_version_hash = dataset_name.split("@", 1)

    if "/" in dataset_name:
        # Example: org1/mnist@3d1e0f91c
        organization_name, dataset_name = dataset_name.split("/", 1)

    dataset = read_dataset(dataset_name, organization_name=organization_name)

    if not dataset.is_version_enabled:
        return ProtoVolumeMountRequest(
            mount_type=MOUNT_TYPE_DATASET,
            mount_path=mount_path,
            dataset=ProtoVolumeMountRequestSourceDataset(
                dataset_id=dataset.id,
                dataset_name=dataset_name,
            ),
        )

    if dataset_version_hash != DATASET_VERSION_HASH_LATEST:
        dataset_version_hash = read_dataset_version(
            dataset.id, dataset_version_hash
        ).version_hash  # Get full version hash

    return ProtoVolumeMountRequest(
        mount_type=MOUNT_TYPE_DATSET_VERSION,
        mount_path=mount_path,
        dataset_version=ProtoVolumeMountRequestSourceDatasetVersion(
            dataset_id=dataset.id,
            dataset_name=dataset_name,
            dataset_version_hash=dataset_version_hash,
        ),
    )


def _configure_volume_mount_request_empty_dir() -> ProtoVolumeMountRequest:
    return ProtoVolumeMountRequest(
        mount_type=MOUNT_TYPE_EMPTY_DIR,
        mount_path=MOUNT_PATH_EMPTY_DIR,
    )


def _configure_volume_mount_request_output(output_dir: str) -> ProtoVolumeMountRequest:
    return ProtoVolumeMountRequest(
        mount_type=MOUNT_TYPE_OUTPUT,
        mount_path=output_dir,
    )


def _configure_volume_mount_request_project(
    local_project_url: str,
    **kwargs,
) -> ProtoVolumeMountRequest:
    project = _get_project(**kwargs)

    volume_mount_project = ProtoVolumeMountRequestSourceProject(
        project_id=project.id,
        project_type=project.type,
    )

    if project.type == PROJECT_TYPE_VERSION_CONTROL:
        volume_mount_project.version_control_project = (
            ProtoVolumeMountRequestSourceVersionControlProject(
                project_branch=get_git_branch(project, **kwargs),
                project_git_ref=get_git_ref(project, **kwargs),
                project_git_diff=get_git_diff_path(project, **kwargs),
            )
        )

    elif project.type == PROJECT_TYPE_CLI_DRIVEN:
        if local_project_url is None:
            # Create zip file in the parent of the current working directory

            cwd = os.path.abspath(os.getcwd())
            zip_path = os.path.abspath(
                os.path.join(os.path.join(cwd, os.pardir), f"{project.name}.zip")
            )
            zipper = Zipper(zip_path, "w")
            zipper.zipdir(cwd, zipper)
            zipper.close()

            uploaded = Uploader.upload(
                local_path=zipper.filename,
                volume_id=project.volume_id,
                remote_path=os.path.basename(zipper.filename),
            )
            local_project_url = uploaded.path

            os.remove(zipper.filename)

        volume_mount_project.cli_driven_project = (
            ProtoVolumeMountRequestSourceCLIDrivenProject(
                local_project=local_project_url,
            )
        )

    return ProtoVolumeMountRequest(
        mount_type=MOUNT_TYPE_PROJECT,
        mount_path=MOUNT_PATH_PROJECT.format(project.name),
        project=volume_mount_project,
    )



import os
import sys

project_root = os.path.dirname(
    os.path.dirname(os.path.dirname(os.path.abspath(__file__)))
)
# this line should be on the top to run it on a local environment as 'python savvihub/cli/_main.py'
sys.path.append(project_root)

import click
import sentry_sdk
from click.decorators import pass_context
from sentry_sdk.integrations.logging import ignore_logger

import savvihub
from savvihub.cli._base import VesslGroup
from savvihub.cli._util import prompt_choices
from savvihub.cli.dataset import cli as dataset_cli
from savvihub.cli.experiment import cli as experiment_cli
from savvihub.cli.kernel_cluster import cli as kernel_cluster_cli
from savvihub.cli.kernel_image import cli as kernel_image_cli
from savvihub.cli.kernel_resource_spec import cli as kernel_resource_spec_cli
from savvihub.cli.model import cli as model_cli
from savvihub.cli.organization import cli as organization_cli
from savvihub.cli.project import cli as project_cli
from savvihub.cli.ssh_key import cli as ssh_key_cli
from savvihub.cli.sweep import cli as sweep_cli
from savvihub.cli.volume import cli as volume_cli
from savvihub.cli.workspace import cli as workspace_cli
from savvihub.util.config import DEFAULT_CONFIG_PATH, VesslConfigLoader
from savvihub.util.exception import (
    InvalidOrganizationError,
    InvalidProjectError,
    InvalidTokenError,
    SavvihubApiException,
)

# Configure Sentry
sentry_sdk.init(
    "https://e46fcd750b3a443fbd5b9dbc970e4ecf@o386227.ingest.sentry.io/5911639",
    traces_sample_rate=1.0,
)
ignore_logger("vessl_logger")


def prompt_organizations() -> str:
    organizations = savvihub.list_organizations()
    organization_count = len(organizations)

    new_organization_string = "Create new organization..."
    choices = [(x.name, i) for i, x in enumerate(organizations)] + [
        (new_organization_string, organization_count)
    ]
    choice = prompt_choices("Default organization", choices)

    if choice == organization_count:
        organization_name = click.prompt("Organization name", type=click.STRING)
        regions = savvihub.vessl_api.region_list_api().regions
        region = prompt_choices("Region", [(x.name, x.value) for x in regions])
        savvihub.create_organization(organization_name, region)
    else:
        organization_name = organizations[choice].name

    return organization_name


@click.command(cls=VesslGroup)
@click.version_option()
@pass_context
def cli(ctx: click.Context):
    savvihub.EXEC_MODE = "CLI"
    ctx.ensure_object(dict)


@cli.group(cls=VesslGroup, invoke_without_command=True)
@click.pass_context
@click.option("-t", "--access-token", type=click.STRING)
@click.option("-o", "--organization", type=click.STRING)
@click.option("-p", "--project", type=click.STRING)
@click.option("-f", "--credentials-file", type=click.STRING)
@click.option("--renew-token", is_flag=True)
def configure(
    ctx,
    access_token: str,
    organization: str,
    project: str,
    credentials_file: str,
    renew_token: bool,
):
    if ctx.invoked_subcommand:
        return

    try:
        savvihub.update_access_token(
            access_token=access_token,
            credentials_file=credentials_file,
            force_update=renew_token,
        )
    except InvalidTokenError:
        savvihub.update_access_token(force_update=True)

    try:
        savvihub.update_organization(
            organization_name=organization, credentials_file=credentials_file
        )
    except InvalidOrganizationError:
        organization_name = prompt_organizations()
        savvihub.update_organization(organization_name)

    try:
        savvihub.update_project(project_name=project, credentials_file=credentials_file)
    except InvalidProjectError:
        projects = savvihub.list_projects()
        if len(projects) == 0:
            return

        project_name = prompt_choices("Default project", [x.name for x in projects])
        savvihub.update_project(project_name)

    print(f"Welcome, {savvihub.vessl_api.user.display_name}!")


@configure.vessl_command()
@click.argument("organization", type=click.STRING, required=False)
def organization(organization: str):
    if organization is None:
        organization = prompt_organizations()
    savvihub.update_organization(organization)
    print(f"Saved to {DEFAULT_CONFIG_PATH}.")


@configure.vessl_command()
@click.argument("project", type=click.STRING, required=False)
def project(project: str):
    savvihub.vessl_api.set_organization()

    if project is None:
        projects = savvihub.list_projects()
        if len(projects) == 0:
            return

        project = prompt_choices("Default project", [x.name for x in projects])
    savvihub.update_project(project)
    print(f"Saved to {DEFAULT_CONFIG_PATH}.")


@configure.command()
def list():
    config = VesslConfigLoader()

    username = ""
    email = ""
    organization = config.default_organization or ""
    project = config.default_project or ""

    if config.access_token:
        savvihub.vessl_api.api_client.set_default_header(
            "Authorization", f"Token {config.access_token}"
        )

        try:
            user = savvihub.vessl_api.get_my_user_info_api()
            username = user.username
            email = user.email
        except SavvihubApiException as e:
            pass

    print(
        f"Username: {username}\n"
        f"Email: {email}\n"
        f"Organization: {organization}\n"
        f"Project: {project}"
    )


cli.add_command(dataset_cli)
cli.add_command(experiment_cli)
cli.add_command(kernel_cluster_cli)
cli.add_command(kernel_image_cli)
cli.add_command(kernel_resource_spec_cli)
cli.add_command(model_cli)
cli.add_command(organization_cli)
cli.add_command(project_cli)
cli.add_command(ssh_key_cli)
cli.add_command(sweep_cli)
cli.add_command(volume_cli)
cli.add_command(workspace_cli)


if __name__ == "__main__":
    cli()



from test.conftest import USE_MOCK

import pytest

import savvihub


@pytest.mark.skipif(USE_MOCK, reason="Does not run if mocking is used.")
class TestModel:
    def test_create_model(self):
        # Do not test because requires completed experiment with no errors
        pass

    def test_list_models(self):
        savvihub.list_models()

    def test_read_model(self):
        # Do not test because requires completed experiment with no errors
        pass

    def test_update_model(self):
        # Do not test because requires completed experiment with no errors
        pass

    def test_delete_model(self):
        # Do not test because requires completed experiment with no errors
        pass



# coding: utf-8

"""
    Aron API

    No description provided (generated by Openapi Generator https://github.com/openapitools/openapi-generator)  # noqa: E501

    The version of the OpenAPI document: 1.0.0
    Generated by: https://openapi-generator.tech
"""


try:
    from inspect import getfullargspec
except ImportError:
    from inspect import getargspec as getfullargspec
import pprint
import re  # noqa: F401
import six

from openapi_client.configuration import Configuration


class ResponseFileMetadata(object):
    """NOTE: This class is auto generated by OpenAPI Generator.
    Ref: https://openapi-generator.tech

    Do not edit the class manually.
    """

    """
    Attributes:
      openapi_types (dict): The key is attribute name
                            and the value is attribute type.
      attribute_map (dict): The key is attribute name
                            and the value is json key in definition.
    """
    openapi_types = {
        'download_url': 'StorageFileActionURLInfo',
        'hash': 'str',
        'is_dir': 'bool',
        'path': 'str',
        'size': 'int',
        'status': 'str',
        'updated_dt': 'datetime',
        'upload_url': 'StorageFileActionURLInfo'
    }

    attribute_map = {
        'download_url': 'download_url',
        'hash': 'hash',
        'is_dir': 'is_dir',
        'path': 'path',
        'size': 'size',
        'status': 'status',
        'updated_dt': 'updated_dt',
        'upload_url': 'upload_url'
    }

    def __init__(self, download_url=None, hash=None, is_dir=None, path=None, size=None, status=None, updated_dt=None, upload_url=None, local_vars_configuration=None):  # noqa: E501
        """ResponseFileMetadata - a model defined in OpenAPI"""  # noqa: E501
        if local_vars_configuration is None:
            local_vars_configuration = Configuration.get_default_copy()
        self.local_vars_configuration = local_vars_configuration

        self._download_url = None
        self._hash = None
        self._is_dir = None
        self._path = None
        self._size = None
        self._status = None
        self._updated_dt = None
        self._upload_url = None
        self.discriminator = None

        if download_url is not None:
            self.download_url = download_url
        self.hash = hash
        self.is_dir = is_dir
        self.path = path
        self.size = size
        self.status = status
        self.updated_dt = updated_dt
        if upload_url is not None:
            self.upload_url = upload_url

    @property
    def download_url(self):
        """Gets the download_url of this ResponseFileMetadata.  # noqa: E501


        :return: The download_url of this ResponseFileMetadata.  # noqa: E501
        :rtype: StorageFileActionURLInfo
        """
        return self._download_url

    @download_url.setter
    def download_url(self, download_url):
        """Sets the download_url of this ResponseFileMetadata.


        :param download_url: The download_url of this ResponseFileMetadata.  # noqa: E501
        :type download_url: StorageFileActionURLInfo
        """

        self._download_url = download_url

    @property
    def hash(self):
        """Gets the hash of this ResponseFileMetadata.  # noqa: E501


        :return: The hash of this ResponseFileMetadata.  # noqa: E501
        :rtype: str
        """
        return self._hash

    @hash.setter
    def hash(self, hash):
        """Sets the hash of this ResponseFileMetadata.


        :param hash: The hash of this ResponseFileMetadata.  # noqa: E501
        :type hash: str
        """
        if self.local_vars_configuration.client_side_validation and hash is None:  # noqa: E501
            raise ValueError("Invalid value for `hash`, must not be `None`")  # noqa: E501

        self._hash = hash

    @property
    def is_dir(self):
        """Gets the is_dir of this ResponseFileMetadata.  # noqa: E501


        :return: The is_dir of this ResponseFileMetadata.  # noqa: E501
        :rtype: bool
        """
        return self._is_dir

    @is_dir.setter
    def is_dir(self, is_dir):
        """Sets the is_dir of this ResponseFileMetadata.


        :param is_dir: The is_dir of this ResponseFileMetadata.  # noqa: E501
        :type is_dir: bool
        """
        if self.local_vars_configuration.client_side_validation and is_dir is None:  # noqa: E501
            raise ValueError("Invalid value for `is_dir`, must not be `None`")  # noqa: E501

        self._is_dir = is_dir

    @property
    def path(self):
        """Gets the path of this ResponseFileMetadata.  # noqa: E501


        :return: The path of this ResponseFileMetadata.  # noqa: E501
        :rtype: str
        """
        return self._path

    @path.setter
    def path(self, path):
        """Sets the path of this ResponseFileMetadata.


        :param path: The path of this ResponseFileMetadata.  # noqa: E501
        :type path: str
        """
        if self.local_vars_configuration.client_side_validation and path is None:  # noqa: E501
            raise ValueError("Invalid value for `path`, must not be `None`")  # noqa: E501

        self._path = path

    @property
    def size(self):
        """Gets the size of this ResponseFileMetadata.  # noqa: E501


        :return: The size of this ResponseFileMetadata.  # noqa: E501
        :rtype: int
        """
        return self._size

    @size.setter
    def size(self, size):
        """Sets the size of this ResponseFileMetadata.


        :param size: The size of this ResponseFileMetadata.  # noqa: E501
        :type size: int
        """
        if self.local_vars_configuration.client_side_validation and size is None:  # noqa: E501
            raise ValueError("Invalid value for `size`, must not be `None`")  # noqa: E501

        self._size = size

    @property
    def status(self):
        """Gets the status of this ResponseFileMetadata.  # noqa: E501


        :return: The status of this ResponseFileMetadata.  # noqa: E501
        :rtype: str
        """
        return self._status

    @status.setter
    def status(self, status):
        """Sets the status of this ResponseFileMetadata.


        :param status: The status of this ResponseFileMetadata.  # noqa: E501
        :type status: str
        """
        if self.local_vars_configuration.client_side_validation and status is None:  # noqa: E501
            raise ValueError("Invalid value for `status`, must not be `None`")  # noqa: E501

        self._status = status

    @property
    def updated_dt(self):
        """Gets the updated_dt of this ResponseFileMetadata.  # noqa: E501


        :return: The updated_dt of this ResponseFileMetadata.  # noqa: E501
        :rtype: datetime
        """
        return self._updated_dt

    @updated_dt.setter
    def updated_dt(self, updated_dt):
        """Sets the updated_dt of this ResponseFileMetadata.


        :param updated_dt: The updated_dt of this ResponseFileMetadata.  # noqa: E501
        :type updated_dt: datetime
        """
        if self.local_vars_configuration.client_side_validation and updated_dt is None:  # noqa: E501
            raise ValueError("Invalid value for `updated_dt`, must not be `None`")  # noqa: E501

        self._updated_dt = updated_dt

    @property
    def upload_url(self):
        """Gets the upload_url of this ResponseFileMetadata.  # noqa: E501


        :return: The upload_url of this ResponseFileMetadata.  # noqa: E501
        :rtype: StorageFileActionURLInfo
        """
        return self._upload_url

    @upload_url.setter
    def upload_url(self, upload_url):
        """Sets the upload_url of this ResponseFileMetadata.


        :param upload_url: The upload_url of this ResponseFileMetadata.  # noqa: E501
        :type upload_url: StorageFileActionURLInfo
        """

        self._upload_url = upload_url

    def to_dict(self, serialize=False):
        """Returns the model properties as a dict"""
        result = {}

        def convert(x):
            if hasattr(x, "to_dict"):
                args = getfullargspec(x.to_dict).args
                if len(args) == 1:
                    return x.to_dict()
                else:
                    return x.to_dict(serialize)
            else:
                return x

        for attr, _ in six.iteritems(self.openapi_types):
            value = getattr(self, attr)
            attr = self.attribute_map.get(attr, attr) if serialize else attr
            if isinstance(value, list):
                result[attr] = list(map(
                    lambda x: convert(x),
                    value
                ))
            elif isinstance(value, dict):
                result[attr] = dict(map(
                    lambda item: (item[0], convert(item[1])),
                    value.items()
                ))
            else:
                result[attr] = convert(value)

        return result

    def to_str(self):
        """Returns the string representation of the model"""
        return pprint.pformat(self.to_dict())

    def __repr__(self):
        """For `print` and `pprint`"""
        return self.to_str()

    def __eq__(self, other):
        """Returns true if both objects are equal"""
        if not isinstance(other, ResponseFileMetadata):
            return False

        return self.to_dict() == other.to_dict()

    def __ne__(self, other):
        """Returns true if both objects are not equal"""
        if not isinstance(other, ResponseFileMetadata):
            return True

        return self.to_dict() != other.to_dict()



import functools
import inspect

import savvihub
from savvihub._version import __VERSION__
from savvihub.util import logger
from savvihub.util.api import VesslApi
from savvihub.util.constant import VESSL_LOG_LEVEL

__version__ = __VERSION__

# Execution mode
EXEC_MODE = "SDK"

# Make API calls using `vessl_api`
vessl_api = VesslApi()


# Expose functions
init = vessl_api.initialize
update_access_token = vessl_api.update_access_token
update_organization = vessl_api.update_default_organization
update_project = vessl_api.update_default_project

from savvihub.dataset import (
    copy_dataset_volume_file,
    create_dataset,
    delete_dataset_volume_file,
    download_dataset_volume_file,
    list_dataset_volume_files,
    list_datasets,
    read_dataset,
    read_dataset_version,
    upload_dataset_volume_file,
)
from savvihub.experiment import (
    create_experiment,
    download_experiment_output_files,
    list_experiment_logs,
    list_experiment_output_files,
    list_experiments,
    read_experiment,
    read_experiment_by_id,
    update_experiment_plots_files,
    update_experiment_plots_metrics,
)
from savvihub.kernel_cluster import (
    delete_cluster,
    list_cluster_nodes,
    list_clusters,
    read_cluster,
    rename_cluster,
)
from savvihub.kernel_image import list_kernel_images, read_kernel_image
from savvihub.kernel_resource_spec import (
    list_kernel_resource_specs,
    read_kernel_resource_spec,
)
from savvihub.model import (
    create_model,
    delete_model,
    list_models,
    read_model,
    update_model,
)
from savvihub.organization import (
    create_organization,
    list_organizations,
    read_organization,
)
from savvihub.project import clone_project, create_project, list_projects, read_project
from savvihub.ssh_key import create_ssh_key, delete_ssh_key, list_ssh_keys
from savvihub.sweep import (
    create_sweep,
    list_sweep_logs,
    list_sweeps,
    read_sweep,
    terminate_sweep,
)
from savvihub.volume import (
    copy_volume_file,
    create_volume_file,
    delete_volume_file,
    list_volume_files,
    read_volume_file,
)
from savvihub.workspace import (
    backup_workspace,
    connect_workspace_ssh,
    list_workspaces,
    read_workspace,
    restore_workspace,
    update_vscode_remote_ssh,
)

# Expose others
from savvihub.integration.common import log  # isort:skip
from savvihub.util.image import Image  # isort:skip


# Suppress all exceptions during SDK mode
def suppress_sdk_exception(f):
    @functools.wraps(f)
    def func(*args, **kwargs):
        try:
            return f(*args, **kwargs)
        except Exception as e:
            if EXEC_MODE == "SDK":
                # Temporarily disable all exceptions that occur within the SDK
                # TODO (VSSL-1992): raise appropriate errors
                exc_info = e if VESSL_LOG_LEVEL == "DEBUG" else False
                logger.exception(f"{e.__class__.__name__}: {str(e)}", exc_info=exc_info)
            else:
                raise e

    return func


# SDK functions that the user may interact with
for name, value in inspect.getmembers(
    savvihub, predicate=lambda x: inspect.ismethod(x) or inspect.isfunction(x)
):
    vars()[name] = suppress_sdk_exception(value)

# Other classes that the user may interact with
user_classes = [Image]
for user_class in user_classes:
    for name, value in inspect.getmembers(
        user_class, predicate=lambda x: inspect.ismethod(x) or inspect.isfunction(x)
    ):
        setattr(user_class, name, suppress_sdk_exception(value))



import os
from pathlib import Path

import toml
from schema import And, Optional, Schema

DEFAULT_VESSL_DIR = os.path.join(str(Path.home()), ".savvihub")
DEFAULT_CONFIG_PATH = os.environ.get(
    "SAVVIHUB_CONFIG_PATH", os.path.join(DEFAULT_VESSL_DIR, "config")
)


class ConfigLoader:
    def __init__(self, filename: str, schema: Schema):
        self.filename = filename
        self.schema = schema
        self.config = {}
        self._load()  # May raise TomlDecodeError
        self._validate()  # May raise SchemaError

    def _load(self):
        if not Path(self.filename).is_file():
            return {}

        with open(self.filename) as f:
            self.config = toml.load(f)

    def _validate(self):
        self.config = self.schema.validate(self.config)

    def save(self):
        self._validate()
        os.makedirs(os.path.dirname(self.filename), exist_ok=True)
        with open(self.filename, "w") as f:
            toml.dump(self.config, f)


class VesslConfigLoader(ConfigLoader):
    schema = Schema(
        {
            Optional("user"): And(
                {
                    Optional("access_token"): str,
                    Optional("default_organization"): str,
                    Optional("default_project"): str,
                    Optional("workspace"): int,
                },
                ignore_extra_keys=True,
            ),
        },
    )

    def __init__(self, path: str = DEFAULT_CONFIG_PATH):
        super().__init__(path, self.schema)

    @property
    def access_token(self):
        return self.config.get("user", {}).get("access_token")

    @access_token.setter
    def access_token(self, access_token: str):
        user = self.config.get("user", {})
        if access_token:
            user["access_token"] = access_token
        self.config["user"] = user
        self.save()

    @property
    def default_organization(self):
        return self.config.get("user", {}).get("default_organization")

    @default_organization.setter
    def default_organization(self, default_organization_name: str):
        user = self.config.get("user", {})
        if default_organization_name:
            user["default_organization"] = default_organization_name
        self.config["user"] = user
        self.save()

    @property
    def default_project(self):
        return self.config.get("user", {}).get("default_project")

    @default_project.setter
    def default_project(self, default_project_name: str):
        user = self.config.get("user", {})
        if default_project_name:
            user["default_project"] = default_project_name
        self.config["user"] = user
        self.save()

    @property
    def workspace(self):
        return self.config.get("user", {}).get("workspace")

    @workspace.setter
    def workspace(self, workspace_slug):
        user = self.config.get("user", {})
        if workspace_slug:
            user["workspace"] = workspace_slug
        self.config["user"] = user
        self.save()



import inspect
import os

import pytest
from mock import Mock, patch

import savvihub
from openapi_client.api.api_v1_api import APIV1Api
from openapi_client.models import (
    SignInAPIPayload,
    SignInCliConfirmAPIPayload,
    SignUpAPIPayload,
)
from savvihub.util.config import VesslConfigLoader
from savvihub.util.exception import SavvihubApiException

USE_MOCK = not os.environ.get("VESSL_TEST_MOCK_OFF", False)
TEST_CONFIG_PATH = "test/config"

savvihub.EXEC_MODE = "TEST"


def pytest_sessionstart(session):
    savvihub.vessl_api.config_loader = VesslConfigLoader(TEST_CONFIG_PATH)

    if USE_MOCK:
        savvihub.vessl_api.organization = Mock()
        savvihub.vessl_api.organization.name = "test-org"
        savvihub.vessl_api.project = Mock()
        savvihub.vessl_api.project.name = "project"
        patch_api_calls()

    else:
        savvihub.vessl_api.api_client.configuration.host = os.environ.get(
            "SAVVIHUB_API_HOST", "http://localhost:10000"
        )
        setup_test_account()


def pytest_sessionfinish(session, exitstatus):
    try:
        os.remove(TEST_CONFIG_PATH)
    except OSError:
        pass


def generate_patch_fixture(name, return_value):
    @pytest.fixture(scope="session", autouse=True)
    def func():
        with patch(name, return_value=return_value) as _fixture:
            yield _fixture

    return func


def patch_api_calls():
    for name, value in inspect.getmembers(APIV1Api, predicate=inspect.isfunction):
        if name.endswith("api"):
            patch_name = f"savvihub.vessl_api.{name}"
            globals()[name] = generate_patch_fixture(patch_name, Mock())


def setup_test_account():
    name = "test-name"
    username = "test-username"
    password = "test-password"
    email = "test@email.com"

    # Setup user
    try:
        jwt_token = savvihub.vessl_api.sign_in_api(
            sign_in_api_payload=SignInAPIPayload(
                email_or_username=email,
                password=password,
            )
        ).token
    except SavvihubApiException:
        jwt_token = savvihub.vessl_api.sign_up_api(
            sign_up_api_payload=SignUpAPIPayload(
                name=name,
                username=username,
                password=password,
                email=email,
                invitation_token="invitation_token_for_cli_test",
            )
        ).token

    # Configure access token
    cli_token = savvihub.vessl_api.sign_in_cli_token_api().cli_token
    savvihub.vessl_api.api_client.set_default_header(
        "Authorization",
        f"JWT {jwt_token}",
    )
    savvihub.vessl_api.sign_in_cli_confirm_api(
        sign_in_cli_confirm_api_payload=SignInCliConfirmAPIPayload(
            cli_token=cli_token,
        )
    )
    access_token = savvihub.vessl_api.sign_in_cli_check_api(
        cli_token=cli_token
    ).access_token
    savvihub.update_access_token(access_token=access_token)

    # Configure organization
    organization_name = "test-org"
    organizations = savvihub.list_organizations()
    if organization_name not in {x.name for x in organizations}:
        savvihub.create_organization(
            organization_name=organization_name,
            region=savvihub.vessl_api.region_list_api().default_region,
        )
    savvihub.update_organization(organization_name=organization_name)

    # Configure project
    project_name = "test-project"
    projects = savvihub.list_projects()
    if project_name not in {x.name for x in projects}:
        savvihub.create_project(
            project_type="cli-driven",
            project_name=project_name,
        )
    savvihub.update_project(project_name=project_name)



from typing import Any, Dict, List, Tuple

from openapi_client.models import (
    InfluxdbSweepLog,
    OrmEarlyStoppingSetting,
    OrmEarlyStoppingSpec,
    OrmEnvVar,
    OrmParameter,
    OrmRange,
    OrmSweepObjective,
    OrmSweepSearchSpace,
    ResponseSweepInfo,
    ResponseSweepListResponse,
    SweepCreateAPIPayload,
)
from savvihub import vessl_api
from savvihub.kernel_cluster import read_cluster
from savvihub.kernel_resource_spec import (
    _configure_custom_kernel_resource_spec,
    read_kernel_resource_spec,
)
from savvihub.organization import _get_organization_name
from savvihub.project import _get_project_name
from savvihub.util.constant import MOUNT_PATH_OUTPUT
from savvihub.volume import _configure_volume_mount_requests


def read_sweep(sweep_name: str, **kwargs) -> ResponseSweepInfo:
    """Read sweep

    Keyword args:
        organization_name (str): override default organization
        project_name (str): override default project
    """
    return vessl_api.sweep_read_api(
        organization_name=_get_organization_name(**kwargs),
        project_name=_get_project_name(**kwargs),
        sweep=sweep_name,
    )


def list_sweeps(**kwargs) -> List[ResponseSweepListResponse]:
    """List sweeps

    Keyword args:
        organization_name (str): override default organization
        project_name (str): override default project
    """
    return vessl_api.sweep_list_api(
        organization_name=_get_organization_name(**kwargs),
        project_name=_get_project_name(**kwargs),
    ).results


def create_sweep(
    objective_type: str,
    objective_goal: float,
    objective_metric: str,
    max_experiment_count: int,
    parallel_experiment_count: int,
    max_failed_experiment_count: int,
    algorithm: str,
    parameters: List[Dict[str, Any]],
    cluster_name: str,
    start_command: str,
    kernel_resource_spec_name: str = None,
    processor_type: str = None,
    cpu_limit: float = None,
    memory_limit: float = None,
    gpu_type: str = None,
    gpu_limit: int = None,
    kernel_image_url: str = None,
    *,
    early_stopping_name: str = None,
    early_stopping_settings: List[Tuple[str, str]] = None,
    message: str = None,
    env_vars: List[Tuple[str, str]] = None,
    dataset_mounts: List[Tuple[str, str]] = None,
    root_volume_size: str = None,
    working_dir: str = None,
    output_dir: str = MOUNT_PATH_OUTPUT,
    local_project_url: str = None,
    **kwargs,
) -> ResponseSweepInfo:
    """Create sweep

    Args:
        parameters (List[Dict[str, Any]]):
            Element keys are 'name' (str), 'type' (str), and 'range' (dict).
            'range' keys are 'list' (list), 'min' (str), 'max' (str),
            and 'step' (str).

    Keyword args:
        organization_name (str): override default organization
        project_name (str): override default project
        git_branch (str): override current git branch
        git_ref (str): override current git commit
        use_git_diff (bool): run experiment with uncommitted changes
        use_git_diff_untracked (bool): run with untracked changed (only valid if `use_git_diff` is set)
    """
    orm_parameters = [
        OrmParameter(
            name=x.get("name"), range=OrmRange(**x.get("range", {})), type=x.get("type")
        )
        for x in parameters
    ]

    cluster = read_cluster(cluster_name)

    kernel_resource_spec = kernel_resource_spec_id = None
    if cluster.is_savvihub_managed:  # TODO: rename to vessl
        kernel_resource_spec_id = read_kernel_resource_spec(
            kernel_resource_spec_name
        ).id
    else:
        kernel_resource_spec = _configure_custom_kernel_resource_spec(
            processor_type,
            cpu_limit,
            memory_limit,
            gpu_type,
            gpu_limit,
        )

    early_stopping_spec = OrmEarlyStoppingSpec(
        algorithm_name=early_stopping_name,
        algorithm_settings=[
            OrmEarlyStoppingSetting(name=k, value=str(v))
            for k, v in (early_stopping_settings or [])
        ],
    )

    volume_mount_requests = _configure_volume_mount_requests(
        dataset_mounts=dataset_mounts,
        root_volume_size=root_volume_size,
        working_dir=working_dir,
        output_dir=output_dir,
        local_project_url=local_project_url,
        **kwargs,
    )

    return vessl_api.sweep_create_api(
        organization_name=_get_organization_name(**kwargs),
        project_name=_get_project_name(**kwargs),
        sweep_create_api_payload=SweepCreateAPIPayload(
            objective=OrmSweepObjective(
                type=objective_type,
                goal=str(objective_goal),
                metric=objective_metric,
            ),
            max_experiment_count=max_experiment_count,
            parallel_experiment_count=parallel_experiment_count,
            max_failed_experiment_count=max_failed_experiment_count,
            algorithm=algorithm,
            search_space=OrmSweepSearchSpace(
                parameters=orm_parameters,
            ),
            cluster_name=cluster_name,
            env_vars=[OrmEnvVar(key, str(value)) for key, value in (env_vars or [])],
            image_url=kernel_image_url,
            early_stopping_spec=early_stopping_spec,
            message=message,
            resource_spec=kernel_resource_spec,
            resource_spec_id=kernel_resource_spec_id,
            start_command=start_command,
            volumes=volume_mount_requests,
        ),
    )


def terminate_sweep(sweep_name: str, **kwargs) -> ResponseSweepInfo:
    """Terminate sweep

    Keyword args:
        organization_name (str): override default organization
        project_name (str): override default project
    """
    return vessl_api.sweep_terminate_api(
        organization_name=_get_organization_name(**kwargs),
        project_name=_get_project_name(**kwargs),
        sweep=sweep_name,
    )


def list_sweep_logs(
    sweep_name: str, tail: int = 200, **kwargs
) -> List[InfluxdbSweepLog]:
    """List sweep logs

    Args:
        tail (int): number of lines to display from the end. Display all if -1.

    Keyword args:
        organization_name (str): override default organization
        project_name (str): override default project
    """

    if tail == -1:
        tail = None

    return vessl_api.sweep_logs_api(
        organization_name=_get_organization_name(**kwargs),
        project_name=_get_project_name(**kwargs),
        sweep=sweep_name,
        limit=tail,
    ).logs



from typing import List

from openapi_client.models import (
    CustomClusterUpdateAPIPayload,
    ResponseKernelClusterInfo,
    ResponseKernelClusterNodeInfo,
)
from savvihub import vessl_api
from savvihub.organization import _get_organization_name


def read_cluster(cluster_name: str, **kwargs) -> ResponseKernelClusterInfo:
    """Read cluster

    Keyword args:
        organization_name (str): override default organization
    """
    return vessl_api.cluster_read_api(
        cluster_name=cluster_name,
        organization_name=_get_organization_name(**kwargs),
    )


def list_clusters(**kwargs) -> List[ResponseKernelClusterInfo]:
    """List clusters

    Keyword args:
        organization_name (str): override default organization
    """
    return vessl_api.all_cluster_list_api(
        organization_name=_get_organization_name(**kwargs),
    ).clusters


def delete_cluster(cluster_name: str, **kwargs) -> object:
    """Delete custom cluster

    Keyword args:
        organization_name (str): override default organization
    """
    return vessl_api.custom_cluster_delete_api(
        cluster_name=cluster_name,
        organization_name=_get_organization_name(**kwargs),
    )


def rename_cluster(
    cluster_name: str, new_cluster_name: str, **kwargs
) -> ResponseKernelClusterInfo:
    """Rename custom cluster

    Keyword args:
        organization_name (str): override default organization
    """
    return vessl_api.custom_cluster_update_api(
        cluster_name=cluster_name,
        organization_name=_get_organization_name(**kwargs),
        custom_cluster_update_api_payload=CustomClusterUpdateAPIPayload(
            name=new_cluster_name,
        ),
    )


def list_cluster_nodes(
    cluster_name: str, **kwargs
) -> List[ResponseKernelClusterNodeInfo]:
    """List custom cluster nodes

    Keyword args:
        organization_name (str): override default organization
    """
    return vessl_api.custom_cluster_node_list_api(
        cluster_name=cluster_name,
        organization_name=_get_organization_name(**kwargs),
    ).nodes
